import java.lang.*;
import java.io.*;
import java.net.*;

/**
 * @author Elizabeth Sklar
 * @author Simon Parsons
 * 
 * Last modified November 16th 2012
 *
 * Simple socket-based client that passes a message typed at the command line.
 *
 * Looks for two command line arguments, the first is a hostname and the second
 * is a port number.
 */
public class Client {
    public static void main( String[] args ) throws IOException {
	
	// Grab the hostname from the command line. Fatal error if one is
	// not provided.
	try{
	    String host;
	    host = args[0];
	
	    // Extract a port number from the command line. Fatal error if one 
	    // is not provided.
	    int port;
	    try{
		port = Integer.parseInt(args[1]);

		try{
		    // Make connection to server socket.
		    Socket server = new Socket(host,port);
		    System.out.println("Connected to server");

		    // Open up streams and wrap them in classes that abstract away
		    // problems in passing data.
		    InputStream inStream = server.getInputStream();
		    OutputStream outStream = server.getOutputStream();
		    BufferedReader in 
			= new BufferedReader(new InputStreamReader(inStream));
		    PrintWriter out 
			= new PrintWriter(new OutputStreamWriter(outStream));

		    // Set up stream to read from the keyboard
		    BufferedReader stdin = 
			new BufferedReader(new InputStreamReader(System.in));

		    // Until the user types "bye", loop, reading from
		    // the keyboard writing the resulting string to
		    // the socket, and reading then printing the ack
		    // from the server.
		    boolean more = true;
		    while(more){
			System.out.println("Type a string, (bye to quit):");
			String msg = (String)stdin.readLine();
			out.println(msg);
			out.flush();
			if(msg.equals("bye")){
			    more = false;
			}
			else{
			    String ack = (String)in.readLine();
			    System.out.println("Server says [" + ack + "]");
			}
		    }
		    // Close the connection
		    stdin.close();
		    inStream.close();
		    outStream.close();
		    server.close();
		} 
		// We have failed to open the connection
		catch (UnknownHostException uhx) {
		    System.err.println(uhx);
		    System.exit(1);
		} 
		catch (IOException iox) {
		    System.err.println(iox);
		    System.exit(1);
		}
	    }
	    // On the client side a missing port is a fatal error
	    catch(ArrayIndexOutOfBoundsException aioobe){
		System.err.println("Proper usage: java Client <host> <port>");
		System.err.println("No port provided, so nothing we can do");
		System.exit(1);
	    }	
	}
	// No hostname provided on the command line
	catch(ArrayIndexOutOfBoundsException aioobe){
	    System.err.println("Proper usage: java Client <host> <port>");
	    System.err.println("No hostname or post provided, so nothing we can do");
	    System.exit(1);
	}
    }
}
