import java.lang.*;
import java.net.*;
import java.io.*;

/**
 * @author Elizabeth Sklar
 * @author Simon Parsons
 * Last modified November 16th 2012
 *
 * Simple socket-based server that receives a string from its clients..
 *
 * When run with no command line parameter the server picks an open port,
 * but a port number can be passed as a paremeter (
 */
public class Server {
    public static void main( String[] args ) throws IOException {

	// Extract a port number from the command line argument if one
	// exists.
	int port;
	try{
	    port = Integer.parseInt(args[0]);
	}
	catch(ArrayIndexOutOfBoundsException aioobe){
	    System.out.println("No port provided, so making one up");
	    port = 0;
	}
	
	// Set up to accept a socket connection on port. If port has value 0
	// the constructor uses a free port.
	try {
	    ServerSocket serverSocket = new ServerSocket(port);

	    // Print out data on where server is listening.
	    System.out.println( "host="+serverSocket.getInetAddress() );
	    System.out.println( "port="+serverSocket.getLocalPort() );
	
	    // Prepare to accept connection from client. The Server will wait at
	    // the serverSocket.accept() command until it receives a request from
	    // the client.
	    try {
		Socket client = serverSocket.accept();
		System.out.println( "Accepted a client!" );

		// Open up streams and wrap them in classes that abstract away
		// problems in passing data.
		InputStream inStream = client.getInputStream();
		OutputStream outStream = client.getOutputStream();
		BufferedReader in 
		    = new BufferedReader(new InputStreamReader(inStream));
		PrintWriter out 
		    = new PrintWriter(new OutputStreamWriter(outStream));

		// In this inner loop we read from the input stream and 
		// respond by writing to the output stream.
		// 
		// Server expects to get strings, and replies with "ok"
		boolean more = true;
		while(more){
		    String input = (String)in.readLine();
		    System.out.println("Client says [" + input + "]");
		    if(input.equals("bye")){
			more = false;
		    }
		    else{
			// Don't acknowledge "bye" because the client
			// will have shutdown the connection
			out.println("Ok");
			out.flush();
		    }
		}
		// Close the connection from the server side
		inStream.close();
		outStream.close();
		client.close();
	    } 
	    // Here we have failed to accept the connection
	    catch ( IOException iox ) {
		System.err.println(iox);
		System.exit(1);
	    }
	}
	// Here we couldn't open the selected port from some reason
	catch ( IOException iox ) {
	    System.err.println( "Could not listen on port: " + port );
	    System.exit( 1 );
	    
	}
    }
}